@@ -12,12 +12,12 @@ from obs.libs import utils
 from requests_aws4auth import AWS4Auth
 from obs.api.app.helpers.rest import response
 from werkzeug.utils import secure_filename
-from flask import request, send_file, current_app
+from flask import request, current_app, send_from_directory
 from flask_restful import Resource, reqparse
 
 
 def get_resources(access_key, secret_key):
-    endpoint = auth.get_endpoint()
+    endpoint = auth.get_endpoint("storage")
     sess = boto3.Session(aws_access_key_id=access_key, aws_secret_access_key=secret_key)
     s3_resource = sess.resource("s3", endpoint_url=endpoint)
     return s3_resource
@@ -28,15 +28,15 @@ def get_plain_auth(access_key, secret_key):
     return auth
 
 
-def list_objects(buckets):
+def list_objects(buckets, contents="Contents", date="LastModified"):
     objects = []
     if buckets["CommonPrefixes"]:
         for prefix in buckets["CommonPrefixes"]:
             objects.append({"directory": f"{prefix['Prefix']}"})
 
-    if buckets["Contents"]:
-        for content in buckets["Contents"]:
-            content["LastModified"] = f'{content["LastModified"]:%Y-%m-%d %H:%M:%S}'
+    if buckets[contents]:
+        for content in buckets[contents]:
+            content[date] = f"{content[date]:%Y-%m-%d %H:%M:%S}"
             objects.append(content)
     return objects
 
@@ -284,7 +284,7 @@ class download_object(Resource):
                     name = archive(args["object_name"])
                 else:
                     name = args["object_name"]
-                file = send_file(f"{tempdir}/{name}", as_attachment=True)
+                file = send_from_directory(tempdir, name, as_attachment=True)
                 return file
             except Exception as e:
                 current_app.logger.error(f"{e}")
@@ -292,6 +292,88 @@ class download_object(Resource):
 
 
 class upload_object(Resource):
+    def get(self, bucket_name):
+        parser = reqparse.RequestParser()
+        parser.add_argument("access_key", type=str, required=True)
+        parser.add_argument("secret_key", type=str, required=True)
+        parser.add_argument("object_name", type=str, default="")
+        parser.add_argument("upload_id", type=str)
+        args = parser.parse_args()
+        secret_key = args["secret_key"].replace(" ", "+")
+
+        try:
+            if args["upload_id"]:
+                parts = bucket.list_part(
+                    get_resources(args["access_key"], secret_key),
+                    bucket_name,
+                    args["object_name"],
+                    args["upload_id"],
+                )
+                if parts is None:
+                    return response(404, f"Part not found, please abort this object.")
+                for part in parts:
+                    part["LastModified"] = f'{part["LastModified"]:%Y-%m-%d %H:%M:%S}'
+                return response(201, data=parts)
+
+            mpu = bucket.list_multipart_upload(
+                get_resources(args["access_key"], secret_key),
+                bucket_name,
+                args["object_name"],
+            )
+            lmpu = list_objects(mpu, "Uploads", "Initiated")
+            return response(200, data=mpu)
+        except Exception as e:
+            current_app.logger.error(f"{e}")
+            return response(500, f"{e}")
+
+    def delete(self, bucket_name):
+        parser = reqparse.RequestParser()
+        parser.add_argument("access_key", type=str, required=True)
+        parser.add_argument("secret_key", type=str, required=True)
+        parser.add_argument("object_name", type=str, required=True)
+        parser.add_argument("upload_id", type=str, required=True)
+        args = parser.parse_args()
+        secret_key = args["secret_key"].replace(" ", "+")
+
+        try:
+            mpu = bucket.abort_multipart_upload(
+                get_resources(args["access_key"], secret_key),
+                bucket_name,
+                args["object_name"],
+                args["upload_id"],
+            )
+            return response(
+                200,
+                f"Multipart upload for {args['object_name']} has been aborted.",
+                mpu,
+            )
+        except Exception as e:
+            current_app.logger.error(f"{e}")
+            return response(500, f"{e}")
+
+    def put(self, bucket_name):
+        parser = reqparse.RequestParser()
+        parser.add_argument("access_key", type=str, required=True)
+        parser.add_argument("secret_key", type=str, required=True)
+        parser.add_argument("object_name", type=str, required=True)
+        parser.add_argument("upload_id", type=str, required=True)
+        args = parser.parse_args()
+        secret_key = args["secret_key"].replace(" ", "+")
+
+        try:
+            mpu = bucket.complete_multipart_upload(
+                get_resources(args["access_key"], secret_key),
+                bucket_name,
+                args["object_name"],
+                args["upload_id"],
+            )
+            return response(
+                200, f"Multipart upload for {args['object_name']} has been completed."
+            )
+        except Exception as e:
+            current_app.logger.error(f"{e}")
+            return response(500, f"{e}")
+
     def post(self, bucket_name):
         parser = reqparse.RequestParser()
         parser.add_argument("access_key", type=str, required=True)
@@ -303,22 +385,19 @@ class upload_object(Resource):
 
         file = request.files["files"]
         filename = secure_filename(file.filename)
-        file.save(filename)
-
         object_name = args["object_name"] if args["object_name"] else filename
 
         try:
             regex = r"[\"\{}^%`\]\[~<>|#]|[^\x00-\x7F]"
             object_name = re.sub(regex, "", object_name)
 
-            result = bucket.upload_object(
+            result = bucket.upload_bin_object(
                 resource=get_resources(args["access_key"], secret_key),
                 bucket_name=bucket_name,
-                local_path=filename,
+                fileobj=file,
                 object_name=object_name,
+                content_type=file.content_type,
             )
-            os.remove(filename)
-
             if args["acl"]:
                 bucket.set_acl(
                     resource=get_resources(args["access_key"], secret_key),
@@ -394,14 +473,16 @@ class acl(Resource):
                 acl_type=acl_type,
                 acl=args["acl"],
             )
-            return response(200, f"Added {acl} access to {acl_type} {name}.", result)
+            return response(
+                200, f"Added {args['acl']} access to {acl_type} {name}.", result
+            )
         except Exception as e:
             current_app.logger.error(f"{e}")
             return response(500, f"{e}")
 
 
 class presign(Resource):
-    def post(self, bucket_name, object_name):
+    def get(self, bucket_name, object_name):
         parser = reqparse.RequestParser()
         parser.add_argument("expire", type=int)
         parser.add_argument("access_key", type=str, required=True)
