@@ -17,5 +17,5 @@ def fake_session(**kwargs):
 def test_resource(monkeypatch):
     monkeypatch.setattr(config, "load_config_file", fake_config)
     monkeypatch.setattr(boto3, "Session", fake_session)
-    monkeypatch.setattr(auth, "get_endpoint", lambda: None)
+    monkeypatch.setattr(auth, "get_endpoint", lambda url: None)
     assert auth.resource() == "s3_resource"
